package com.dragon.tj.portal.component.message;

import com.alibaba.fastjson.JSON;
import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.service.TokenService;
import com.dragon.tj.portal.common.base.R;
import com.dragon.tj.portal.common.dto.message.MessageInfoReq;
import com.dragon.tj.portal.service.MessageInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;

/**
 * @author huey China.
 * @Description : 控制
 * @Date Created in 2023/6/15 15:43
 */
@Slf4j
@RestController
public class SseController {

    @Resource
    private Map<String, OpenApiSseEmitter> sseEmitters;

    @Autowired
    private MessageInfoService messageInfoService;

    @Autowired
    private TokenService tokenService;

    @GetMapping("/subscribe/{clientId}")
    public SseEmitter subscribe(@PathVariable String clientId, HttpServletResponse response) {

        response.addHeader("Cache-Control", "no-cache");
        response.addHeader("X-Accel-Buffering", "no");
        response.setHeader("Access-Control-Allow-Origin", "*");

        log.info("got a subscription, clientId={} will be bound to serverId={}", clientId, "appId");
        // 创建一个新的SseEmitter对象并设置超时时间，并将其存储在Map中
        OpenApiSseEmitter emitter = new OpenApiSseEmitter(Long.MAX_VALUE);
        sseEmitters.put(clientId, emitter);
        emitter.onCompletion(() -> sseEmitters.remove(clientId));

        return emitter;
    }

    @PostMapping("/publish")
    public R publish(@Validated @RequestBody MessageInfoReq messageInfoReq, HttpServletRequest request) {
        log.info("sseController req param is {}", JSON.toJSONString(messageInfoReq));
        LoginUser loginUser = tokenService.getLoginUser(request);
        log.info("sseController current people is {}", JSON.toJSONString(loginUser));
        return messageInfoService.push(messageInfoReq,loginUser);
    }


}
