package com.dragon.tj.portal.controller;

import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.util.SecurityUtils;
import com.dragon.tj.portal.common.base.R;
import com.dragon.tj.portal.entity.AppInfo;
import com.dragon.tj.portal.entity.PageParam;
import com.dragon.tj.portal.service.AppService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

@RestController
@RequestMapping("/app")
@EnableScheduling
public class AppController {


    @Autowired
    private AppService appService;

    /*
     *    1. 我的应用查询 批量排序收藏
     * */
    @PostMapping("list")
    public R appList(@RequestBody PageParam<AppInfo> appInfoPage) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return R.ok(appService.installApp(appInfoPage,loginUser));
    }

    /*
     *  1-1安装应用
     * */
    @GetMapping("install/{appId}")
    public R install(@PathVariable("appId") Integer appId) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return R.ok(appService.install(appId,loginUser.getIdCard()));
    }

    /*
     *  1-2卸载应用
     * */
    @GetMapping("uninstall/{appId}")
    public R uninstall(@PathVariable("appId") Integer appId) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return R.ok(appService.uninstall(appId,loginUser.getIdCard()));
    }

    /***************************************/

    /*
     *    2. 类型查询
     * */
    @GetMapping("types")
    public R getAppType() {
        return appService.getType();
    }

    /*
     *    3. 应用 CRUD
     * */
//    @SysLog("添加应用")
    @PostMapping("add")
    public R addApp(@RequestBody AppInfo appInfo) {
        System.out.println(appInfo);
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return R.ok(appService.add(appInfo, loginUser));
    }

    //    @SysLog("应用删除")
    @GetMapping("delete/{id}")
    public R delete(@PathVariable("id") Long id) {
        return R.ok(appService.delete(id));
    }

    //    @SysLog("应用更新")
    @PostMapping("update")
    public R update(@RequestBody AppInfo appInfo) {
        appInfo.setUpdateTime(LocalDateTime.now());
        return R.ok(appService.update(appInfo));
    }

    //    @SysLog("通过ID获取应用")
    @GetMapping("info/{id}")
    public R detail(@PathVariable("id") Long id) {
        return R.ok(appService.detail(id));
    }

    /*
     *   通过应用名称,应用事权单位名称搜索app
     * */
    @GetMapping("search")
    public R search(@RequestBody PageParam<AppInfo> appInfoPage) {

        return R.ok(appService.search(appInfoPage));
    }


    /*
     *    4. 热门应用统计 定时、实时？
     * */
    @Scheduled(fixedDelay = 1000 * 60 * 60)
    public void appStatHandler(){

        appService.updateAppStat();
    }

    /*
     *    5. 专项工具为应用类型的一种
     * */
    /**********************/

    /*
    * 上传图片
    * */



}

