package com.dragon.tj.portal.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dragon.tj.portal.auth.model.LoginUser;
import com.dragon.tj.portal.auth.util.SecurityUtils;
import com.dragon.tj.portal.common.util.UploadUtils;
import com.dragon.tj.portal.entity.FileInfo;
import com.dragon.tj.portal.entity.PageParam;
import com.dragon.tj.portal.mapper.FileManageMapper;
import com.dragon.tj.portal.service.FileManageService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;

@Service
public class FileManageServiceImpl implements FileManageService {

    @Value("${dragon.file.path}")
    private String localPath;

    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private FileManageMapper fileManageMapper;

    @Override
    public FileInfo upload(MultipartFile file,Integer type) {
        String originalFilename = file.getOriginalFilename();
        System.out.println(originalFilename);

        String uuidFilename = UploadUtils.getUUIDName(originalFilename);

        File dateDir = new File(localPath + type);
        if (!dateDir.exists()) {
            dateDir.mkdirs();
        }

        File newFile = new File(dateDir.getPath(),uuidFilename);

        try {
            file.transferTo(newFile);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        Map<String,String> res = new HashMap<>();
        FileInfo fileInfo = new FileInfo();
        String[] split = originalFilename.split("\\.");
        fileInfo.setFileName(split[0]);
        fileInfo.setExtension(split[1]);
        fileInfo.setSize(file.getSize());
        fileInfo.setType(type);
        fileInfo.setContentType(file.getContentType());
        LoginUser loginUser = SecurityUtils.getLoginUser();
        fileInfo.setCreateUser(loginUser.getUsername());

        try {
            String hostAddress = InetAddress.getLocalHost().getHostAddress();
            String port = applicationContext.getEnvironment().getProperty("server.port");
            String url = "http://"+ hostAddress + ":" + port + "/file/" + type + "/" +uuidFilename;
            fileInfo.setUrl(url);
        } catch (UnknownHostException e) {
            e.printStackTrace();
        }

        //save to db
        fileManageMapper.insert(fileInfo);

        return fileInfo;
    }

    @Override
    public Page<FileInfo> search(PageParam<FileInfo> pageParam) {
        FileInfo fileParam = pageParam.getParams();
        Page<FileInfo> rowPage = new Page(pageParam.getPage(), pageParam.getSize());

        LambdaQueryWrapper<FileInfo> queryWrapper = new LambdaQueryWrapper<>();

        //文件分类category(办公常用、系统工具)
        queryWrapper.eq(fileParam.getType() != null,FileInfo::getType, fileParam.getType());
        queryWrapper.eq(FileInfo::getDelFlag, 0);

        //文件名称
        queryWrapper.like(StringUtils.isNotEmpty(fileParam.getFileName()), FileInfo::getFileName, fileParam.getFileName());
        //文件拓展名
        queryWrapper.like(StringUtils.isNotEmpty(fileParam.getExtension()), FileInfo::getExtension, fileParam.getExtension());
        //ContentType
        queryWrapper.like(StringUtils.isNotEmpty(fileParam.getContentType()), FileInfo::getContentType, fileParam.getContentType());


        //下载量排序
        queryWrapper.orderByDesc(FileInfo::getDownloads);

        return fileManageMapper.selectPage(rowPage, queryWrapper);
    }

    @Override
    public List<FileInfo> top(String type, Integer num) {
        List<String> list = Arrays.asList(type.split(","));
        return fileManageMapper.getTop(list,num);
    }
}
